# Generated by roxygen2: do not edit by hand

S3method(default_values,LearnerClassifRanger)
S3method(default_values,LearnerClassifSVM)
S3method(default_values,LearnerClassifXgboost)
S3method(default_values,LearnerRegrRanger)
S3method(default_values,LearnerRegrSVM)
S3method(default_values,LearnerRegrXgboost)
export(LearnerClassifCVGlmnet)
export(LearnerClassifGlmnet)
export(LearnerClassifKKNN)
export(LearnerClassifLDA)
export(LearnerClassifLogReg)
export(LearnerClassifMultinom)
export(LearnerClassifNaiveBayes)
export(LearnerClassifNnet)
export(LearnerClassifQDA)
export(LearnerClassifRanger)
export(LearnerClassifSVM)
export(LearnerClassifXgboost)
export(LearnerRegrCVGlmnet)
export(LearnerRegrGlmnet)
export(LearnerRegrKKNN)
export(LearnerRegrKM)
export(LearnerRegrLM)
export(LearnerRegrNnet)
export(LearnerRegrRanger)
export(LearnerRegrSVM)
export(LearnerRegrXgboost)
import(checkmate)
import(data.table)
import(mlr3misc)
import(paradox)
importFrom(R6,R6Class)
importFrom(methods,formalArgs)
importFrom(mlr3,LearnerClassif)
importFrom(mlr3,LearnerRegr)
importFrom(mlr3,assert_quantiles)
importFrom(mlr3,assert_validate)
importFrom(mlr3,mlr_learners)
importFrom(stats,predict)
importFrom(stats,reformulate)
importFrom(utils,bibentry)
importFrom(utils,packageVersion)
useDynLib(mlr3learners,c_ranger_mu_sigma)
useDynLib(mlr3learners,c_ranger_var)
